import pandas as  pd
import matplotlib.pyplot as plt
import numpy as np

"""
1.
Read in the auto-mpg.csv and save it to a variable called auto. Check
that auto is a pandas dataframe
"""
auto = pd.read_csv('auto-mpg.csv')
type(auto) # pandas dataframe


"""
2.
Understand what the data set consists of. Is every value of the name field unique?
If so, we can use it as the index. If not, find at least one example of duplicate
values in the name field
"""
"""
Each row in the data set is a different automobile. For each one, we have its
name, the year, and several metrics about the vehicle such as miles per gallon,
number of cylinders, horsepower, etc.
"""
auto.info() 
"""
The name field is not unique. There are 392 rows in the data set, but only 301
unique names.
"""
print("number of rows in data:",auto.shape[0])
print("number of unique values of name", auto['name'].nunique())

"""
We can find duplicate names with groupby:
"""
name_counts = auto.groupby(['name']).size()
name_counts[name_counts > 1]

"""
Any name in the above output is an example of a non uniquely occuring name.
"""
auto.loc[auto['name'] == 'chevrolet chevette']

#%%
"""
3.
For this exercise we will look at the relationship between horsepower and
weight. First, make a scatterplot of the two variables, with horsepower on
the x axis and weight on the y axis. Be sure to provide x and y labels, and
a title
"""
fig, ax = plt.subplots(1,1)
ax.scatter(auto['hp'], auto['weight'])
ax.set_xlabel('Horsepower')
ax.set_ylabel('Weight')
ax.set_title('Weight vs Horsepower')


#%%
"""
4.
The plot shows one point that is quite far from the rest. It is good practice
to identify and understand any data points that seem to not fit the trend.
Using boolean masks or DataFrame.query(), try to find this value. When you
find it, create a variable called outlier that contains the row number
in the data set where this car is located. 
Note: It will be easiest if outlier has type Series. If it has a DataFrame 
type, convert it to a series by taking out the row you want (using iloc for
example).
"""
outlier = auto.query('hp > 200 and weight < 3500').iloc[0]
type(outlier)
#%%
"""
5. 
Add an annotation near the outlier. The annotation should say the car's name,
year, and number of cylinders it has.
You may want to change the alignment of the annotation. You can use the
argument horizontalalignment='right' and verticalalignment='top'
"""
label_text = '{} {}\n ({} cyl.)'.format(outlier['yr'], outlier['name'], outlier['cyl'])
ax.text(outlier['hp'], outlier['weight'], label_text,
        horizontalalignment='right', verticalalignment='top')


#%%
"""
6.
Make a boxplot of the weight and horsepower variables. Make them as 
two subplots of the same figure. Use a subplot grid with two rows
and one column. It will look best to use horizontal boxplots with this layout.
Should you use any of the `sharex` / `sharey` options?
Make sure to provide an x label and title for each of the subplots.

"""

fig, ax = plt.subplots(2,1)
"""
We do not need to share x or y axes, because the variables have different
units. It doesn't make sense to use the same scale for units of horsepower
and weight.
"""
ax[0].boxplot(auto['weight'], vert=False)
ax[0].set_xlabel('Weight')
ax[0].set_title('Weight distribution')

ax[1].boxplot(auto['hp'], vert=False)
ax[1].set_xlabel('Horsepower')
ax[1].set_title('Horsepower distribution')


#%%

"""
7.
Are there outliers for either of these variables? If so, what value can you 
use as a threshold to determine outliers (this can be approximate, just read 
it from the boxplot). Make a filtered data frame that just contains the outliers.
Call it hp_outliers. Print the name and year of these cars.
"""

"""
There are no outliers for weight, but there are a handful of outliers for
horsepower. The cutoff point is around hp=200.
"""

hp_outliers = auto.query('hp > 200')
num_outliers = len(hp_outliers)
for i in range(num_outliers):
    print(hp_outliers.iloc[i]['yr'], ' ', hp_outliers.iloc[i]['name'])


#%%
"""
8.
Create a scatter plot of weight vs. horsepower using the full auto data set.
Then on the same plot, plot the outliers data frame in red.
"""
fig, ax = plt.subplots(1,1)
ax.scatter(auto['hp'], auto['weight'])
ax.scatter(hp_outliers['hp'], hp_outliers['weight'], c='r')

#%%
"""
9.
What do you notice about a lot of the outliers?
"""

"""
They may have been outliers for horsepower (ie very large values), but a lot
of them do fit the trend when plotted against weight. We will learn another
method for finding outliers that takes the trend into account.
"""


#%%
"""
10.
For now we'll exclude the outliers. Make a data frame using just the 
non-outlier auto data, and call it auto_cleaned. Make a scatterplot with
this data frame.
"""

auto_cleaned = auto.query('hp <= 200')
fix, ax = plt.subplots(1,1)
ax.scatter(auto_cleaned['hp'], auto_cleaned['weight'])
#%%
"""
11.
Now we would like to compute the best fit line of this data. You will learn 
more about this in your linear class, but we can start to familiarize ourselves
now. 
A straight line requires two values: the y-intercept and the slope. We seek
to find a "line of best fit," which typically is defined to mean that the 
line we draw will minimize the total "difference" between the points and the line.
There are many ways to define what we mean by "difference," but in most cases, 
the definition is to: for each x value, compare the y value(s) that appears 
in the data, and compare this to the correspopnding y valueon the best-fit line.
Take the difference of these two numbers and square them. Do this for every x
value, and then take the total of the squared error. We seek to find a line
that minimizes this *total sum of squared error*.

Fortunately there is a formula to do just this! The y intercept of the best-fit
line can be calculated
as:
    $
    $
and the slope as:
    $
    $

The capital greek letter sigma indicates a sum from i to N, the number of data
points. Think of a for loop from i to N: everything inside the sum is like
being inside of the for loop. 


Write a function to compute beta0 and beta1 from your x and y values (horsepower
and weight). Make sure to use the auto_cleaned data set.
"""
def beta(x,y):
    y_avg = y.mean()
    x_avg = x.mean()
    beta1_num = sum((x - x_avg) * (y - y_avg))
    beta1_den = sum((x - x_avg)**2)
    beta1 = beta1_num / beta1_den

    beta0 = y_avg - beta1*x_avg
    
    return (beta0, beta1)

hp = auto_cleaned['hp']
weight = auto_cleaned['weight']
beta0, beta1 = beta(hp, weight)
#%%
"""
12.
Now plot the same scatter plot and include your best fit line. Add an
x label, y label, and title. Also add an annotation giving the equation of 
the best fit line.
"""
xgrid = np.linspace(min(hp), max(hp), 100)
bestfit = beta0 + beta1*xgrid

fig, ax = plt.subplots(1,1)
ax.scatter(hp, weight)
ax.plot(xgrid, bestfit, 'k')
ax.set_xlabel('Horsepower')
ax.set_ylabel('Weight')
ax.set_title('Relationship between weight and horsepower')
bf_eq = '$w = {} + {}\cdot hp$'.format(round(beta0,2), round(beta1,2)) 
ax.text(45, 4500, bf_eq)

#%%
"""
13.
Save your plot as bestfit.png
"""
fig.set_size_inches(6, 6)
plt.savefig('bestfit.png')
