import numpy as np
# import matplotlib.pyplot as plt

np.random.seed(100)

u = 60
sig = 5
speeds = np.random.normal(u, sig, 4000)
# plt.hist(speeds, 30)
N = len(speeds)

# part 1
def p_right(x):
    return sum(speeds >= x)/N

print('p_right for 70', p_right(70))
print('p_right for 70', p_right(60))
print('p_right for 85', p_right(85))

"""
According to our calculation, a very large speed like 85 seems to 
have 0 probability of occuring! This is of course not true. The
reason we are getting 0 is because there are no observations of 85
in our sample. 85 is very unlikely (but not 0), so we would probably
need to draw many more observations before we observed it.
"""

# part 2
def p_left(x):
    return sum( speeds <= x) / N


print('p_left for 50', p_left(58))
print('p_left for 40', p_left(50))
print('p_left for 30', p_left(40))

"""
By the same reasoning, there are no observations of a speed of 40
in our sample. If we drew a much larger sample, then we would gradually 
start to see values that low, but they are very unlikely.
"""

# part 3
p1sig = p_right(u+sig) + p_left(u-sig)
p2sig = p_right(u+2*sig) + p_left(u-2*sig)
p3sig = p_right(u+3*sig) + p_left(u-3*sig)


"""
We've calculated the two-tailed p-values for 1, 2, and 3 standard
deviations. This gives the probability that a value would be outside
+/- 1, 2 or 3 sigma. If we subtract that probability from 1, we get the 
probability that a value will fall inside the range. This is called
the confidence interval.
"""
print('p1sig', p1sig, 1-p1sig)
print('p2sig', p2sig, 1-p2sig)
print('p3sig', p3sig, 1-p3sig)
