#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 25 11:32:12 2017

@author: brentskoumal
"""

###Intro
"""
=============================================================================
For this exercise we are going to be reading in the same file from Assignment 1
(smoking.csv). However, this time we will take advantage of Python's powerful 
Pandas library. If you recall, the file contains data about the economic impact 
of smoking in different states, and for every year from 2005-2009. 

We've included this file in your course materials. It's called smoking.csv. 

In this exercise, you will again be asked to fix problems with the .csv, and 
then perform some computations on the data.

The assignment consists of four parts, shown below. Your responses will 
either be in the form of code or a short answer. Simply edit this file directly
to include your responses. You should be able to run this file without
causing any errors.
"""

#%%
###Step 1.
"""
If you would like to refresh your memory about the dataset it can be found here

    https://data.world/health/smoking-attributables-sae

View the data by clicking the green CSV button (left side about halfway down). 
"""
#%%
###Step 2.
"""
Import the Pandas package (using an import statement) and rename the package as "pd" for ease of use.

Using Pandas, read the .csv file to a Pandas DataFrame object, store it as a variable "df" and examine the first five lines of this DataFrame. Notice that the ".read_csv()" method has many optional arguments that it accepts. You can tell pandas to skip lines, parse dates, handle missing values, etc. 

To view the documentation on the read_csv method, type "pd.read_csv" into the help tab (in the top right corner of Spyder)

After the DataFrame has been created, you can view it in an "Excel-style" window by simply double-clicking the DataFrame in the Variable Explorer window (top-right side of Spyder)
"""

import pandas as pd
df = pd.read_csv('smoking.csv')
df.head()

#%%
###Step 3.
"""
There are a number of useful DataFrame attributes that Pandas provides. 
Refer to the Pandas DataFrame documentation here:
    
https://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.html

Notice that attributes are different than methods, attributes don't require parentheses. 

Use Pandas DataFrame attributes to:
    Check the DataFrame column names
    Check the DataFrame index
    Check the DataFrame values
    Check the DataFrame data types
    Check the DataFrame size
    Check the DataFrame shape
"""

print("Columns of df are: \n",df.columns)
print("--------------------------")
print("Index of df is: \n",df.index)
print("--------------------------")
print("Values of df are: \n",df.values)
print("--------------------------")
print("Dtypes of df are: \n",df.dtypes)
print("--------------------------")
print("Size of df is: \n",df.size)
print("--------------------------")
print("Shape of df is: \n",df.shape)

#%%
###Step 4.
"""
Use Pandas DataFrame methods to describe the DataFrame summary statistics (notice that Pandas ignores non-numeric columns, if you want to see the summary stats for all columns, pass include='all')
"""
df.describe()
#%%
###Step 5.
"""
Total up (sum) the "null" values using Pandas methods
"""
df.isnull().sum()
#%%
###Step 6.
"""
From the last step, we can see that Data_Value_Footnote_Symbol and Data_Value_Footnote both have over 1000 missing values, use the .drop() method to remove them from the DataFrame (remember to pass "inplace=True" so that the original df is actually modified)
"""
df.drop(['Data_Value_Footnote_Symbol','Data_Value_Footnote'],axis=1,inplace=True)
df
#%%
###Step 7.
"""
Pandas' .read_csv() method implies a numeric index unless you specify one, you can always reset the index though. Set the DataFrame index to the "year" column (again, remember to pass "inplace=True" so that the original df is actually modified)
"""
df.set_index('Year', inplace=True)
df
#%%
###Step 8.
"""
Pandas also provides a number of options for slicing/dicing DataFrames. Let's create a new DataFrame that only contains the data that we care about, and since it looks like "Data_Value_Footnote_Symbol", "Data_Value_Footnote", "Topic Type ID", "Topic ID", "Measure ID", "SubMeasureID", and "DisplayOrder" aren't useful to us, use .loc or .iloc to select only the columns that we care about, and assign them to a new DataFrame called "df2".

"""
df2 = df.loc[:,:'Data_Value']
df2
#alternate solution
#df2 = df.iloc[:,:10]
#%%
###Step 9.
"""
Since "DataSource", "TopicType", "TopicDesc", "MeasureDesc","Data_Value_Unit", and "Data_Value_Type" all have one and only one value, they aren't very useful either. Let's also drop them from the DataFrame, but this time create a list of these column names and pass that to the .drop() method.
"""
drop_list = ["DataSource", "TopicType", "TopicDesc", "MeasureDesc","Data_Value_Unit", "Data_Value_Type"]

df2.drop(drop_list,axis=1,inplace=True)
df2

#%%
#Step 10.
"""
Use the Pandas .groupby() method to tally-up the "Data_Value for each of the states in df2, assign the resulting dataframe to a new dataframe called df3
"""
    
df3 = df2.groupby('LocationDesc').sum()
#alternate solution
#df3 = df2.groupby('LocationAbbr').sum()

#%%
#Step 11.
"""
Which state has the highest total?
Hint: Use the .sort_values() method
"""
df3.sort_values(by='Data_Value', ascending=False)